@@ -1,6 +1,6 @@
-use num::ToPrimitive;
-use num_rational::Ratio;
 use paste::paste;
+use rust_decimal::prelude::ToPrimitive;
+use rust_decimal::Decimal;
 
 use std::borrow::Cow;
 use std::fmt;
@@ -9,6 +9,9 @@ use crate::parse::token::Field as ParsedField;
 use crate::parse::token::Value as ParsedValue;
 
 #[derive(Clone, PartialEq, Debug)]
+/// The output struct for gcode emission implementing [std::fmt::Display]
+///
+/// Any strings here are expected to have escaped characters, see <https://www.reprap.org/wiki/G-code#Quoted_strings>
 pub enum Token<'a> {
     Field(Field<'a>),
     Comment {
@@ -38,7 +41,9 @@ impl fmt::Display for Token<'_> {
     }
 }
 
-/// Fundamental unit of GCode: a value preceded by a descriptive letter.
+/// Fundamental unit of GCode: a descriptive letter followed by a value.
+///
+/// Field type supports owned and partially-borrowed representations using [Cow].
 #[derive(Clone, PartialEq, Debug)]
 pub struct Field<'a> {
     pub letters: Cow<'a, str>,
@@ -62,21 +67,38 @@ impl<'input> From<&ParsedField<'input>> for Field<'input> {
 
 impl<'a> From<Field<'a>> for Token<'a> {
     fn from(field: Field<'a>) -> Token<'a> {
-        Token::Field(field)
+        Self::Field(field)
+    }
+}
+
+impl<'a> Field<'a> {
+    /// Returns an owned representation of the Field valid for the `'static` lifetime.
+    ///
+    /// This will allocate any string types.
+    pub fn into_owned(self) -> Field<'static> {
+        Field {
+            letters: self.letters.into_owned().into(),
+            value: self.value.into_owned(),
+        }
     }
 }
 
 /// All the possible variations of a field's value.
 /// Some flavors of GCode also allow for strings.
+///
+/// Any strings here are expected to have escaped characters, see <https://www.reprap.org/wiki/G-code#Quoted_strings>
 #[derive(Clone, PartialEq, Debug)]
 pub enum Value<'a> {
-    Rational(Ratio<i64>),
+    Rational(Decimal),
     Float(f64),
     Integer(usize),
     String(Cow<'a, str>),
 }
 
 impl Value<'_> {
+    /// Interpret the value as an [f64]
+    ///
+    /// Returns [Option::None] for [Value::String] or a [Value::Rational] that can't be converted.
     pub fn as_f64(&self) -> Option<f64> {
         match self {
             Self::Rational(r) => r.to_f64(),
@@ -85,6 +107,18 @@ impl Value<'_> {
             Self::String(_) => None,
         }
     }
+
+    /// Returns an owned representation of the Value valid for the `'static` lifetime.
+    ///
+    /// This will allocate a string for a [Value::String].
+    pub fn into_owned(self) -> Value<'static> {
+        match self {
+            Self::String(s) => Value::String(s.into_owned().into()),
+            Self::Rational(r) => Value::Rational(r),
+            Self::Integer(i) => Value::Integer(i),
+            Self::Float(f) => Value::Float(f),
+        }
+    }
 }
 
 impl<'input> From<&ParsedValue<'input>> for Value<'input> {
@@ -93,7 +127,12 @@ impl<'input> From<&ParsedValue<'input>> for Value<'input> {
         match val {
             Rational(r) => Self::Rational(*r),
             Integer(i) => Self::Integer(*i),
-            String(s) => Self::String(Cow::Borrowed(s)),
+            String(s) => {
+                // Remove enclosing quotes
+                Self::String(Cow::Borrowed(
+                    s.strip_prefix('"').unwrap().strip_suffix('"').unwrap(),
+                ))
+            }
         }
     }
 }
@@ -101,7 +140,16 @@ impl<'input> From<&ParsedValue<'input>> for Value<'input> {
 impl fmt::Display for Value<'_> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            Self::Rational(r) => write!(f, "{}", r.to_f64().ok_or(fmt::Error)?),
+            Self::Rational(r) => {
+                write!(f, "{}", r)?;
+                // The only way this could've been interpreted
+                // as rational is if there is a trailing decimal point,
+                // so add it back in.
+                if r.fract().is_zero() {
+                    write!(f, ".")?;
+                }
+                Ok(())
+            }
             Self::Float(float) => write!(f, "{}", float),
             Self::Integer(i) => write!(f, "{}", i),
             Self::String(s) => write!(f, "\"{}\"", s),
@@ -110,20 +158,25 @@ impl fmt::Display for Value<'_> {
 }
 
 /// A macro for quickly instantiating a float-valued command
+///
+/// For instance:
+///
+/// ```
+/// use g_code::command;
+/// assert_eq!(command!(RapidPositioning { X: 0., Y: 1., }).iter().fold(String::default(), |s, f| s + &f.to_string()), "G0X0Y1");
+/// ```
 #[macro_export]
 macro_rules! command {
     ($commandName: ident {
         $($arg: ident : $value: expr,)*
     }) => {
         {
-            use g_code::emit::*;
-            use paste::paste;
             paste::expr!{
-                [<$commandName:snake:lower>](
+                g_code::emit::[<$commandName:snake:lower>](
                     vec![$(
-                        Field {
-                            letters: Cow::Borrowed(stringify!([<$arg:upper>])),
-                            value: Value::Float($value),
+                        g_code::emit::Field {
+                            letters: std::borrow::Cow::Borrowed(stringify!([<$arg:upper>])),
+                            value: g_code::emit::Value::Float($value),
                         }
                     ,)*].drain(..)
                 )
@@ -138,6 +191,8 @@ macro_rules! impl_commands {
         paste! {
             $(
                 $(#[$outer])*
+                ///
+                /// Call this function to instantiate the command.
                 pub fn [<$commandName:snake:lower>]<'a, I: Iterator<Item = Field<'a>>>(args: I) -> Command<'a> {
                     Command {
                         name: [<$commandName:snake:upper _FIELD>].clone(),
@@ -149,14 +204,17 @@ macro_rules! impl_commands {
                         }).collect(),
                     }
                 }
+
+                /// Constant for this command's name used to reduce allocations.
                 pub const [<$commandName:snake:upper _FIELD>]: Field<'static> = Field {
-                    letters: Cow::Borrowed($letters),
-                    value: Value::Integer($value),
+                    letters: std::borrow::Cow::Borrowed($letters),
+                    value: crate::emit::Value::Integer($value),
                 };
             )*
         }
 
         /// Commands are the operational unit of GCode
+        ///
         /// They consist of a G, M, or other top-level field followed by field arguments
         #[derive(Clone, PartialEq, Debug)]
         pub struct Command<'a> {
@@ -165,32 +223,40 @@ macro_rules! impl_commands {
         }
 
         impl<'a> Command<'a> {
-            pub fn push(&mut self, arg: Field<'a>) {
+            /// Add a field to the command.
+            ///
+            /// Returns [Err] if the Field's letters aren't recognized.
+            pub fn push(&mut self, arg: Field<'a>) -> Result<(), ()> {
                 match &self.name {
-                    $(x if *x == paste!{[<$commandName:snake:upper _FIELD>]}.clone() => {
-                        if match arg.letters.to_ascii_uppercase().as_str() {
-                            $(stringify!($arg) => {true},)*
+                    $(x if *x == paste!{[<$commandName:snake:upper _FIELD>]} => {
+                        if match arg.letters.as_ref() {
+                            $(stringify!([<$arg:upper>]) => {true},)*
+                            $(stringify!([<$arg:lower>]) => {true},)*
                             _ => false,
                         } {
                             self.args.push(arg);
+                            Ok(())
                         } else {
+                            Err(())
                         }
                     },)*
                     _ => {
-                        dbg!(&self.name);
-                        dbg!(&arg);
+                        unreachable!("a command's name cannot change");
                     }
                 }
             }
 
+            /// Iterate over all fields including the command's name (i.e. G0 for rapid positioning)
             pub fn iter(&self) -> impl Iterator<Item = &Field> {
                 std::iter::once(&self.name).chain(self.args.iter())
             }
 
+            /// Consumes the command to produce tokens suitable for output
             pub fn into_token_vec(mut self) -> Vec<Token<'a>> {
                 std::iter::once(self.name).chain(self.args.drain(..)).map(|f| f.into()).collect()
             }
 
+            /// Iterate over the fields after the command's name
             pub fn iter_args(&self) -> impl Iterator<Item = &Field> {
                 self.iter().skip(1)
             }
